#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <mpi.h>

// Crea un array di numeri random tra 0 e 1
double *create_rand_nums(int num_elements) {
  double *rand_nums = (double*) malloc(sizeof(double) * num_elements);
  int i;
  for (i = 0; i < num_elements; i++) {
    rand_nums[i] = (rand() / (double) RAND_MAX);
  }
  return rand_nums;
}

// Calcola la media di un array
double compute_avg(double *array, int num_elements) {
  double sum = 0.0;
  int i;
  for (i = 0; i < num_elements; i++) {
    sum += array[i];
  }
  return sum / num_elements;
}

int main(int argc, char** argv) {
  int root = 0;
  int num_elements_per_proc = 50000;
  // Seed random
  srand(time(NULL));

  MPI_Init(NULL, NULL);

  int world_rank;
  MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
  int world_size;
  MPI_Comm_size(MPI_COMM_WORLD, &world_size);

  // Crea l'array di numeri random (solo se sei il root). 
  // La dimensione è data dal numero di elementi per processo per il numero di core
  double *rand_nums = NULL;
  if (world_rank == root) {
    rand_nums = create_rand_nums(num_elements_per_proc * world_size);
  }

  // Buffer che conterrà la porzione di array
  double *sub_rand_nums = (double*) malloc(sizeof(double) * num_elements_per_proc);

  MPI_Scatter(rand_nums, num_elements_per_proc, MPI_DOUBLE, sub_rand_nums,
              num_elements_per_proc, MPI_DOUBLE, root, MPI_COMM_WORLD);

  // Calcola la media della sottoporzione
  double sub_avg = compute_avg(sub_rand_nums, num_elements_per_proc);

  // Tramite la funzione gather invia le soluzioni parziali alla root
  double *sub_avg_list = NULL;
  if (world_rank == root) {
    sub_avg_list = (double*) malloc(sizeof(double) * world_size);
  }
  MPI_Gather(&sub_avg, 1, MPI_DOUBLE, sub_avg_list, 1, MPI_DOUBLE, root, MPI_COMM_WORLD);

  // Calcola la media finale
  if (world_rank == root) {
    double avg = compute_avg(sub_avg_list, world_size);
    printf("La media del risultati parziali è %f\n", avg);
    // Per verifica calcola la media sui dati originali
    double original_data_avg = compute_avg(rand_nums, num_elements_per_proc * world_size);
    printf("Media sui dati originali %f\n", original_data_avg);
  }

  if (world_rank == root) {
    free(rand_nums);
    free(sub_avg_list);
  }
  free(sub_rand_nums);

  MPI_Finalize();
}