#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <time.h>

// Crea un array di numeri random tra 0 e 1
double *create_rand_nums(int num_elements) {
  double *rand_nums = (double*) malloc(sizeof(double) * num_elements);
  for (int i = 0; i < num_elements; i++) {
    rand_nums[i] = (rand() / (double) RAND_MAX);
  }
  return rand_nums;
}

int main(int argc, char** argv) {
  int root = 0;
  int num_elements_per_proc = 50000;

  MPI_Init(NULL, NULL);

  int world_rank;
  MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
  int world_size;
  MPI_Comm_size(MPI_COMM_WORLD, &world_size);

  // Creao un array random su tutti i processi
  srand(time(NULL) * world_rank);   // Risultato diverso per ogni processo
  double *rand_nums = NULL;
  rand_nums = create_rand_nums(num_elements_per_proc);

  // Calcola la somma locale
  double local_sum = 0;
  for (int i = 0; i < num_elements_per_proc; i++) {
    local_sum += rand_nums[i];
  }

  printf("Somma locale per il processo %d - %f, media = %f\n",
    world_rank, local_sum, local_sum / num_elements_per_proc);

  // Riduci le somme locale nella somma totale
  double global_sum;
  MPI_Reduce(&local_sum, &global_sum, 1, MPI_DOUBLE, MPI_SUM, root, MPI_COMM_WORLD);

  // Stampa del risultato
  MPI_Barrier(MPI_COMM_WORLD);
  if (world_rank == root) {
    printf("Somma totale = %f, media = %f\n", global_sum, global_sum / 
        (world_size * num_elements_per_proc));
  }

  free(rand_nums);
  MPI_Finalize();
}